@@ -13,9 +13,9 @@ from pstats import Stats
 from jinja2 import Template
 
 
-__version__ = '0.1.1'
-__author__ = 'Hideo Hattori <syobosyobo@gmail.com>'
-__licence__ = "New BSD License"
+__version__ = '0.1.2'
+__author__  = 'Hideo Hattori <syobosyobo@gmail.com>'
+__licence__ = 'New BSD License'
 
 __all__ = ['Converter']
 
@@ -47,7 +47,6 @@ span.header-title {
     float: left;
 }
 span.header-time {
-    /*font-family: Arial, Verdana;*/
     text-align: right;
     width: 50%;
     margin-right: 1em;
@@ -148,19 +147,20 @@ XHTML_BODY_TEMPLATE = u"""\
        Total Func Call: <span style="font-weight:bold">{{ profdata['totalcalls'] }}</span> function calls</p>
     <table class="prof">
         <tr>
-        <th class="procheader" align="center" colspan="6">Top {{ profdata['data']|length }} functions data</th>
+            <th class="procheader" align="center" colspan="6">Top {{ profdata['data']|length }} functions data</th>
         </tr>
         <tr align="right">
-        <th class="prof"><span class="tdmg">ncalls</span></th>
-        <th class="prof"><span class="tdmg">tottime</span></th>
-        <th class="prof"><span class="tdmg">percall</span></th>
-        <th class="prof"><span class="tdmg">cumtime</span></th>
-        <th class="prof"><span class="tdmg">percall</span></th>
-        <th class="prof" align="center"><span class="tdmg">filename:lineno(function)</span></th>
+            <th class="prof"><span class="tdmg" title="for the number of calls">ncalls</span></th>
+            <th class="prof"><span class="tdmg" title="for the total time spent in the given function (and excluding time made in calls to sub-functions)">tottime</span></th>
+            <th class="prof"><span class="tdmg" title="the quotient of tottime divided by ncalls">percall</span></th>
+            <th class="prof"><span class="tdmg" title="total time spent in this and all subfunctions (from invocation till exit). This figure is accurate even for recursive functions.">cumtime</span></th>
+            <th class="prof"><span class="tdmg" title="the quotient of cumtime divided by primitive calls">percall</span></th>
+            <th class="prof" align="center"><span class="tdmg" title="provides the respective data of each function">filename:lineno(function)</span></th>
         </tr>
         {% for item in profdata['data'] %}
             <tr>
-            <td class="prof" align="right"><span class="tdmg">{{ item['ncalls'] }}</span></td>
+            <td class="prof" style="background-color:{{ item['ncallslevel'] }}" align="right">
+              <span class="tdmg">{{ item['ncalls'] }}</span></td>
             <td class="prof" style="background-color:{{ item['tottimelevel'] }}" align="right">
               <span class="tdmg">{{ item['tottime'] }}</span></td>
             <td class="prof" style="background-color:{{ item['totcalllevel'] }}" align="right">
@@ -233,6 +233,13 @@ def check_hotshot(dump):
     signature = "hotshot-version"
     return signature in dump
 
+def mapping_table(target, all):
+    level_list = [all*0.01, all*0.03, all*0.08, all*0.12, all*0.20,]
+    for level in range(len(level_list)):
+        if target < level_list[level]:
+            break
+    return level
+
 
 class Converter:
     """output to HTML from profile data"""
@@ -255,10 +262,11 @@ class Converter:
         self.outputtype = 'html'
         self.functions_number = 20
         self.levelmap = {
-            0: '',
-            1: '#fa9cb8',
-            2: '#e95464',
-            3: '#e2041b',
+            0: '#ccffcc',   ## (like) light cyan
+            1: '#66ff99',   ## (like) light cyan
+            2: '#ffcc33',   ## (like) lime
+            3: '#ff6666',   ## yellow
+            4: '#ff3333',   ## red
         }
 
     def _printhtml_onesource(self, filename, profinfo):
@@ -314,7 +322,7 @@ class Converter:
                 if info != {}:
                     filecnt += 1
                     if self.outputtype == 'html':
-                        print self._printhtml_onesource(source_list[filecnt-1],
+                        print >> self.outfile, self._printhtml_onesource(source_list[filecnt-1],
                                                         info)
                     else:
                         print self._print_onesource(source_list[filecnt-1], info)
@@ -322,7 +330,7 @@ class Converter:
                 source_list.append(filename)
             info[prof[1]] = prof[2]
         if self.outputtype == 'html':
-            print self._printhtml_onesource(filename, info)
+            print >> self.outfile, self._printhtml_onesource(filename, info)
         else:
             print self._print_onesource(filename, info)
 
@@ -374,18 +382,21 @@ class Converter:
                 cumpercall = "%8.4lf" % float(stat[3]/stat[0])
             except ZeroDivisionError:
                 cumpercall = "0.0000"
-            tottlevel = self.levelmap[int((stat[2]/self.prof.total_tt)*3.33)]
-            tmp = int((float(totpercall)/self.prof.total_tt)*3.33)
-            totclevel = self.levelmap[tmp]
-            cumtlevel = self.levelmap[int((stat[3]/self.prof.total_tt)*3.33)]
-            tmp = int((float(cumpercall)/self.prof.total_tt)*3.33)
-            cumclevel = self.levelmap[tmp]
+            ncallslevel = self.levelmap[mapping_table(ncalls,  self.prof.total_calls)]
+            tottlevel   = self.levelmap[mapping_table(stat[2], self.prof.total_tt)]
+            totclevel   = self.levelmap[mapping_table(float(totpercall), self.prof.total_tt)]
+            #cumtlevel   = self.levelmap[mapping_table(stat[3], self.prof.total_tt)]
+            #cumclevel   = self.levelmap[mapping_table(float(cumpercall), self.prof.total_tt)]
+            #totclevel   = self.levelmap[0]
+            cumtlevel   = self.levelmap[0]
+            cumclevel   = self.levelmap[0]
             data = {
                 'ncalls'        : ncalls,
                 'tottime'       : tottime,
                 'cumtime'       : cumtime,
                 'totpercall'    : totpercall,
                 'cumpercall'    : cumpercall,
+                'ncallslevel'   : ncallslevel,
                 'cumtimelevel'  : cumtlevel,
                 'tottimelevel'  : tottlevel,
                 'totcalllevel'  : totclevel,
@@ -402,15 +413,15 @@ class Converter:
 
     def _hookhtml(self):
         profdata = self._analyzed_prof()
-        print XHTML_HEADER_TEMPLATE
-        print self.tmpl.render(title="pyprof2html - %s" % self.filename,
+        print >> self.outfile, XHTML_HEADER_TEMPLATE
+        print >> self.outfile, self.tmpl.render(title="pyprof2html - %s" % self.filename,
                                proftime=self.proftime,
                                reporttime=time.ctime(),
                                profdata=profdata)
         if self.proftype:
             profinfo = self._analyze_profline()
             self._print_source(profinfo)
-        print XHTML_FOOTER_TEMPLATE
+        print >> self.outfile, XHTML_FOOTER_TEMPLATE
 
     def printout(self, filetype='html'):
         """print to html or text."""
@@ -419,6 +430,10 @@ class Converter:
         if filetype == 'raw':
             hookpoint = self._hookraw
         else:
+            if not os.path.exists(self.output_dir):
+                os.makedirs(self.output_dir)
+            filepath = self.output_dir + '/index.html'
+            self.outfile = open(filepath, 'w')
             hookpoint = self._hookhtml
         hookpoint()
 
@@ -436,6 +451,9 @@ def main():
                       help='jinja2 template file')
     parser.add_option('-n', '--num', type='int', dest='print_functions',
                       default=20, help='print to N funcs.(default 20)')
+    parser.add_option('-o', '--output-dir', dest='output_dir', default='html',
+                      help='output HTML under the this directory.' \
+                           '(default: html/')
     opts, args = parser.parse_args()
     if not len(args):
         parser.parse_args(['-h'])
@@ -450,6 +468,7 @@ def main():
         outmode = 'html'
     else:
         outmode = 'html'
+    p2h.output_dir = opts.output_dir
     p2h.printout(outmode)
 
 
